%This is a supplementary code for the paper "Local Dynamics in Trained
%Recurrent Neural Networks" by Alexander Rivkind and Omri Barak, 2016

function [GOL,rho]=mftTrainSolve(fp_vec,nwkParam)

g=nwkParam.g;
wden=nwkParam.wden;
phi=nwkParam.phi;
phip=nwkParam.phip;

    %%%%%%%%%vector initializations
    CMX=zeros(length(fp_vec));
    ccMX=zeros(length(fp_vec));
    sigmaVec=zeros(length(fp_vec),1);
    GRsqr=zeros(length(fp_vec),1);
    GnmMX=zeros(length(fp_vec))*tf([0],1);


   %% Calculating correlation matrix
    for kk=1:length(fp_vec)
        for mm=kk:-1:1
            if kk==mm
                sigmaVec(kk)=mftTrainSolveSigma(fp_vec(kk),nwkParam);
                ccMX(kk,mm)=1;
                GRsqr(kk)=g^2*gaussIntWin(@(z,w) phip(w*fp_vec(kk)+z*sigmaVec(kk)).^2,0,1,wden);
                [alphaVec(kk),beta0vec(kk),beta1vec(kk)]=mftTrainSolveAlpha(fp_vec(kk),nwkParam,sigmaVec(kk));
                GnmMX(kk,kk)=alphaVec(kk)*sigmaVec(kk).^2;
            else
                [ccMX(kk,mm)]=mftTrainSolveCorr(fp_vec(kk),fp_vec(mm),nwkParam,sigmaVec(kk),sigmaVec(mm));
                ccMX(mm,kk)=ccMX(kk,mm);
            end
        end
    end

        %% Calculating coefficients \alpha and obtaining gain
    for kk=1:length(fp_vec)
        for mm=1:length(fp_vec)
            if kk~=mm
                [GnmMX(kk,mm)]=mftTrainSolveGnm(fp_vec(kk),fp_vec(mm),nwkParam,...
                    ccMX(kk,mm),sigmaVec(kk),sigmaVec(mm),alphaVec(kk));
            end
        end
    end
    
    %% Computing loop gain
    CMX=ccMX.*(sigmaVec*sigmaVec.');
    k=inv(CMX/g^2)*fp_vec';
    GOL=minreal(g^(-2)*zpk(-1,[],1)*(GnmMX*k),1e-3);
    
    %% Spectral radius
    
    rho=sqrt(GRsqr);