%This is a supplementary code for the paper "Local Dynamics in Trained
%Recurrent Neural Networks" by Alexander Rivkind and Omri Barak, 2016
%
%This is a top script, calling to a function that performs the MFT calculations

%% Initializations
clear;

omega_vec=logspace(-2,2,40);
settingFromPaper='1d'; %pick a figure from paper 1a 1b 1c 1d b
% settingFromPaper=0; %use 0 for manual parameter setting,
doNumSim=1; %toggles to do or not to do numerical simulation

%or use figure reference for a setting from paper

%% network parameters
if settingFromPaper(1)
    [A_vec,nwkParam]=mftTrainPickFig(settingFromPaper);
else
    A_vec=[0.5 1 1.5]; %fixed points to train for
    nwkParam.g=1.5; %network gain parameter
    %activation function selection
    nwkParam.phi=@tanh;
    nwkParam.phip=(@(x) 1-tanh(x).^2);
    %other phi/phi-prime pairs   
    %     nwkParam.phi=(@(x) max(0,x-0.1));
    %     nwkParam.phip=(@(x) x>0.1);
    %     nwkParam.phi=@(x)  1 ./ (1 + exp(-x));
    %     nwkParam.phip=@(x)  exp(-x) ./ (1 + exp(-x)).^2;
    %feedback weight distribution function
    %only supports distributions between -1 and 1
    nwkParam.wden=@(x)((x>=-1)&(x<=1))/2;
    %use a number rather than a function to set constant weights
    %this saves a lot of integration time
    %  nwkParam.wden=1;
    nwkParam.N=1000; %used only for numerical simulation (if turned on)
end

%% Solving the MFT
%Call for the solver function
disp('Computing the MFT solution');
[GOL,rho]=mftTrainSolve(A_vec,nwkParam);
% GCL=cellfun(@(x) x./(1-x),GOL);
for ii=1:length(A_vec)
    GCL(ii)=GOL(ii)/(1-GOL(ii));
end

%% Simulating numerically, if enabled

if doNumSim
    disp('Computing the numerical solution')
    [GOLsim,specSim,failFlag]=mftNumericalSim(A_vec,nwkParam,omega_vec);
    GCLsim=cellfun(@(x) x./(1-x),GOLsim,'UniformOutput',0);
end

%% Plotting Outputs
% G_OL
figure;
hold on;
for ii=1:length(A_vec)
    GOLvOmega{ii}=freqresp(GOL(ii),omega_vec');
    plot(GOLvOmega{ii}(:),'color', 0.8*([1 2 3]==ii))
end
if doNumSim
    for ii=1:length(A_vec)
        plot(GOLsim{ii}(:),'x','color', 0.8*([1 2 3]==ii))
    end
end
grid;
legend(num2str(A_vec'));
xlabel('Re(G^{OL})');
ylabel('Im(G^{OL})');
title('G^{OL}(\omega)');

%GCL
figure;
for ii=1:length(A_vec)
    GCLvOmega{ii}=freqresp(GCL(ii),omega_vec');
    semilogx(omega_vec,abs(GCLvOmega{ii}(:)),'color', 0.8*([1 2 3]==ii));
    hold on;
end
grid;
legend(num2str(A_vec'));
if doNumSim
    for ii=1:length(A_vec)
        semilogx(omega_vec,abs(GCLsim{ii}(:)),'x','color', 0.8*([1 2 3]==ii));
    end
end

xlabel('\omega');
ylabel('|(G^{CL}|');
title('G^{CL}(\omega)');

%Spectrum
figure;
%plotting outliers (MFT)
for ii=1:length(A_vec)
    pGcl{ii}=pole(minreal(GCL(ii),1e-3));
    plot(real(pGcl{ii}),imag(pGcl{ii}),'o','linewidth',2,'markersize',8,'color', 0.8*([1 2 3]==ii));
    hold on;
end
grid;
legend(num2str(A_vec'));
%plotting the border of the continous density disk (MFT)
for ii=1:length(A_vec)
    circleC(-1, 0, rho(ii),0.8*([1 2 3]==ii));
end

%plotting numerical results
if doNumSim
    for ii=1:length(A_vec)
        mySpectrum=   specSim{ii};
        plot(mySpectrum,'.', 'MarkerSize',3,'color', 0.5*([1 2 3]==ii));
        %emphasizing outliers with larger dots
        plot(1i*1e-6+mySpectrum(abs(mySpectrum+1)>rho(ii)),'.', 'MarkerSize',8,'color', 0.5*([1 2 3]==ii))
    end
end

xlabel('Re{\lambda}');
ylabel('Im{\lambda}');
title('Spectrum of the closed loop system');

